use crate::hittable::{HitRecord, Hittable};
use crate::{Aabb, Ray, Vec3};

pub struct Translate<H: Hittable> {
    hittable: H,
    offset: Vec3
}

impl<H: Hittable> Translate<H> {
    pub fn new(hittable: H, offset: Vec3) -> Self {
        Self { hittable, offset }
    }
}

impl<H: Hittable> Hittable for Translate<H> {
    fn hit(&self, ray: &Ray, t_min: f64, t_max: f64) -> Option<HitRecord> {
        let moved_ray = Ray::new(
            ray.origin() - self.offset,
            ray.direction(),
            ray.time());
        match &self.hittable.hit(&moved_ray, t_min, t_max) {
            Some(hit_record) => {
                let mut hr = HitRecord {
                    point: hit_record.point,
                    material: hit_record.material,
                    front_face: hit_record.front_face, // Maybe need to calc normal and front_face again?
                    normal: hit_record.normal,
                    t: hit_record.t,
                    u: hit_record.u,
                    v: hit_record.v
                };
                hr.normalized(ray);
                Some(hr)
            }
            _ => None
        }
    }

    fn bounding_box(&self, time0: f64, time1: f64) -> Option<Aabb> {
        match &self.hittable.bounding_box(time0, time1) {
            Some(bounding_box) => Some(Aabb {
                minimum: bounding_box.minimum + self.offset,
                maximum: bounding_box.maximum + self.offset,
            }),
            _ => None
        }
    }
}
